
import os.path, shutil

class WebsiteBuilder(object):
    def __init__(self, app, localURL, config):
        self.app = app
        self.localURL = localURL
        self.config = config
    
    def build(self):
        appCachePath = self.appCachePath
        if not os.path.exists(appCachePath):
            os.makedirs(appCachePath)
        
        from .PageMaker import PageMaker
        pageMaker = PageMaker(self.app, self.localURL, '/podcasts/', True)
        html = pageMaker.make()
        
        htmlPath = os.path.join(appCachePath, 'index.html')
        f = file(htmlPath, 'w')
        f.write(html)
        f.close()

        staticRoots = {}
        self.collectRelativeStaticRoot(self.app.favicon, staticRoots)
        self.collectRelativeStaticRoot(self.app.touchIcon, staticRoots)
        self.collectRelativeStaticRoot(self.app.startupImage, staticRoots)
        [self.collectStaticRoot(name, staticRoots) for name in pageMaker.styles]
        
        for rootName in staticRoots:
            staticPath = self.app.loader.searchStatic(rootName)
            if staticPath:
                destPath = os.path.join(appCachePath, 'static', rootName)
                if not os.path.exists(destPath):
                    destParent = os.path.dirname(destPath)
                    if not os.path.exists(destParent):
                        os.makedirs(destParent)
                    os.symlink(staticPath, destPath)
        
    def collectStaticRoot(self, name, staticRoots):
        rootName = name.split('/')[0]
        staticRoots[rootName] = 1
    
    def collectRelativeStaticRoot(self, name, staticRoots):
        if name:
            path = self.app.loader.normalizeName(name, self.app.moduleName)
            self.collectStaticRoot(path, staticRoots)
            
    def deploy(self):
        action = self.config.get('action')                                       
        if action == 'rsync':
            appCachePath = self.appCachePath
            appDestPath = self.config.get('destination')

            rsyncCommand = 'rsync -Lavzu%(dry)s -e ssh %(from)s/* %(to)s/'
            command = rsyncCommand % {"dry": "", "from": appCachePath, "to": appDestPath}
            print command
            os.system(command)
    
    @property
    def appCachePath(self):
        cachePath = os.path.expanduser('~/Library/Caches/Appjs')
        return os.path.join(cachePath, 'website', self.app.moduleName)
        