#!/usr/bin/env python

import os.path

usage = """usage: %prog [options] ...

Runs a server for a single app."""

def parseOptions():
    import optparse
    parser = optparse.OptionParser(usage)
    parser.set_defaults(host='', port=80)

    parser.add_option("", "--port", dest="port", type="int",
        help = "Web server port")

    parser.add_option("", "--host", dest="host", type="str",
        help = "Web server host name")

    options, arguments = parser.parse_args()
    
    return [arguments[0] if arguments else '', options.host, options.port]

def main():
    appPath, host, port = parseOptions()

    jspath = os.environ.get('JSPATH', '').split(':')
    jspaths = [os.path.abspath(os.path.expanduser(path)) for path in jspath]

    appjsPath = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
    if appjsPath not in jspaths:
        jspaths.append(appjsPath)

    from Loader import Loader
    loader = Loader(jspaths)
    # XXXjoe Take this list as a command line argument
    loader.transientExtensions = ['.jss']
    
    app = None
    if appPath:
        app = loader.searchApp(appPath)
        if not app:
            import sys
            sys.stderr.write('Unable to find app "%s"\n' % appPath)
            return
    
    from WebServer import WebServer
    WebServer.serve(host, port, loader, app)

if __name__ == '__main__':
    main()
